package com.scbfkj.uni.process;

import com.scbfkj.uni.library.DataFormatUtil;
import com.scbfkj.uni.library.UniReturnUtil;
import org.apache.commons.pool2.BaseKeyedPooledObjectFactory;
import org.apache.commons.pool2.KeyedObjectPool;
import org.apache.commons.pool2.PooledObject;
import org.apache.commons.pool2.impl.DefaultPooledObject;
import org.apache.commons.pool2.impl.GenericKeyedObjectPool;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.converter.StringHttpMessageConverter;
import org.springframework.util.StringUtils;
import org.springframework.web.client.RestTemplate;

import java.nio.charset.StandardCharsets;
import java.util.*;

public class Web {

    private KeyedObjectPool<String, Web> webPool = new GenericKeyedObjectPool<>(new BaseKeyedPooledObjectFactory<String, Web>() {
        @Override
        public Web create(String s) throws Exception {
            return new Web();
        }

        @Override
        public PooledObject<Web> wrap(Web web) {
            return new DefaultPooledObject<>(web);
        }
    });


    private RestTemplate restTemplate; //webapi请求对象


    private String webapiURL;

    private HttpHeaders webApiHeader;
    private String webapiMethod;

    private Map<String, Object> baseHeader = new HashMap<>();

    /**
     * webapi 执行入口函数
     */
    public Map<String, Object> exec(Object header, String method, Object defaultBody, String connectConfig) throws Exception {
        Web web = null;
        try {
            web = webPool.borrowObject(connectConfig);
            return web.execWebApi(header, method, defaultBody, connectConfig);

        } finally {
            if (web != null) {
                webPool.returnObject(connectConfig, web);
            }
        }
    }
    public Map<String, Object> execWebApi(Object header, String method, Object defaultBody, String connectConfig) {
        Map<String, Object> connectConfigMaps = (Map<String, Object>) DataFormatUtil.toMap(connectConfig);
        Map<String, Object> restTemplateResult = initWebApiParams(connectConfigMaps); // 通过连接配置 获取restTemplate

        if (!"0".equals(restTemplateResult.get("code"))) {
            return UniReturnUtil.fail(restTemplateResult.get("message").toString());
        }
        if (header instanceof Map) {
            Map<String, Object> temeMap = (Map) header;
            if (!temeMap.isEmpty()) {
                temeMap.putAll(baseHeader);
                webApiHeader.clear();
                for (String key : temeMap.keySet()) {
                    webApiHeader.add(key, temeMap.get(key).toString());
                }
            }
        }
        if (StringUtils.hasLength(method)) webapiMethod = method.toLowerCase();
        try {
            List<Map<String, Object>> returnList = new ArrayList<>();
            if (defaultBody instanceof List<?> tempList) {
                for (Object o : tempList) {
                    returnList.add(sendWebApiRequest(o));
                }
            } else {
                returnList.add(sendWebApiRequest(defaultBody));
            }
            return (defaultBody instanceof List<?>) ? UniReturnUtil.success(returnList) : returnList.get(0);
        } catch (Exception e) {
            return UniReturnUtil.fail("restTemplate执行异常" + UniReturnUtil.getMessage(e));
        }
    }

    private Map<String, Object> sendWebApiRequest(Object defaultBody) {
        try {
            HttpEntity<Object> request = new HttpEntity<>(defaultBody, webApiHeader);
            String responseEntity = null;
            switch (webapiMethod) {
                case "get" -> responseEntity = restTemplate.getForObject(webapiURL, String.class, request);
                case "put" -> restTemplate.put(webapiURL, request);
                case "delete" -> restTemplate.delete(webapiURL, request);
                default -> responseEntity = restTemplate.postForObject(webapiURL, request, String.class);
            }
            if (responseEntity == null && (webapiMethod.equals("put") || webapiMethod.equals("delete"))) {
                return UniReturnUtil.success(null);
            }
            if (Objects.nonNull(responseEntity)) {
                return UniReturnUtil.success(responseEntity);
            } else {
                return UniReturnUtil.fail("webapi发送请求失败 : " + responseEntity);
            }
        } catch (Exception e) {
            return UniReturnUtil.fail("发送请求失败 : ".concat(UniReturnUtil.getMessage(e)));
        }
    }

    private Map<String, Object> initWebApiParams(Map<String, Object> connectConfig) {
        try {
            String configStr = DataFormatUtil.toString(connectConfig);

            if (restTemplate == null) {
                if (Objects.isNull(connectConfig)) {
                    return UniReturnUtil.fail("webapi请求配置参数为空");
                }
                webapiURL = Objects.nonNull(connectConfig.get("url")) ? connectConfig.get("url").toString() : null;
                if (Objects.isNull(webapiURL)) {
                    return UniReturnUtil.fail("webapi请求地址为空");
                }
                Object tempHeaders = connectConfig.get("headers"); //需确定返回的是一个map对象还是字符串  headers: {ContentType:"JSON"}
                webApiHeader = new HttpHeaders();
                if (Objects.isNull(tempHeaders)) {
                    webApiHeader.setContentType(MediaType.APPLICATION_JSON);
                }
                if (tempHeaders instanceof Map) {
                    baseHeader = (Map) tempHeaders;
                    if (!baseHeader.isEmpty()) {
                        for (String key : baseHeader.keySet()) {
                            webApiHeader.add(key, baseHeader.get(key).toString());
                        }
                    }
                }
                Object webMethod = connectConfig.get("method"); //需确定返回的是一个map对象还是字符串  headers: {ContentType:"JSON"}
                if (Objects.nonNull(webMethod)) {
                    webapiMethod = webMethod.toString().toLowerCase();
                } else {
                    webapiMethod = "post";
                }
                restTemplate = new RestTemplate();
                restTemplate.getMessageConverters().set(1, new StringHttpMessageConverter(StandardCharsets.UTF_8));
            }
        } catch (Exception e) {
            return UniReturnUtil.fail(UniReturnUtil.getMessage(e));
        }
        return UniReturnUtil.success(null);
    }
}
